/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.eviction;

import org.jboss.cache.config.ConfigurationException;
import org.jboss.cache.config.Dynamic;

/**
 * Configuration for {@link ElementSizeAlgorithm}.
 * <p/>
 * Requires a positive "maxElementsPerNode" value otherwise a ConfigurationException is thrown.
 *
 * @author Manik Surtani
 * @since 3.0
 */
public class ElementSizeAlgorithmConfig extends EvictionAlgorithmConfigBase
{
   /**
    * The serialVersionUID
    */
   private static final long serialVersionUID = 2510593544656833758L;

   @Dynamic
   private int maxElementsPerNode;

   public ElementSizeAlgorithmConfig()
   {
      evictionAlgorithmClassName = ElementSizeAlgorithm.class.getName();
      // Force configuration of maxElementsPerNode
      setMaxElementsPerNode(-1);
   }

   public ElementSizeAlgorithmConfig(int maxNodes, int maxElementsPerNode)
   {
      this();
      setMaxNodes(maxNodes);
      setMaxElementsPerNode(maxElementsPerNode);
   }

   public int getMaxElementsPerNode()
   {
      return maxElementsPerNode;
   }

   public void setMaxElementsPerNode(int maxElementsPerNode)
   {
      testImmutability("maxElementsPerNode");
      this.maxElementsPerNode = maxElementsPerNode;
   }

   /**
    * Requires a positive maxElementsPerNode value or ConfigurationException
    * is thrown.
    */
   @Override
   public void validate() throws ConfigurationException
   {
      super.validate();
      if (maxElementsPerNode < 0)
      {
         throw new ConfigurationException("maxElementsPerNode must be must be " +
               "configured to a value greater than or equal to 0");
      }
   }

   @Override
   public String toString()
   {
      StringBuilder str = new StringBuilder();
      str.append("ElementSizeConfiguration: maxElementsPerNode =");
      str.append(getMaxElementsPerNode()).append(" maxNodes =").append(getMaxNodes());
      return str.toString();
   }

   @Override
   public boolean equals(Object obj)
   {
      if (this == obj)
         return true;
      if (obj instanceof ElementSizeAlgorithmConfig && super.equals(obj))
      {
         return this.maxElementsPerNode == ((ElementSizeAlgorithmConfig) obj).maxElementsPerNode;
      }
      return false;
   }

   @Override
   public int hashCode()
   {
      int result = super.hashCode();
      result = 31 * result + maxElementsPerNode;
      return result;
   }

   @Override
   public void reset()
   {
      super.reset();
      setMaxElementsPerNode(-1);
      evictionAlgorithmClassName = ElementSizeAlgorithm.class.getName();
   }

   @Override
   public ElementSizeAlgorithmConfig clone() throws CloneNotSupportedException
   {
      return (ElementSizeAlgorithmConfig) super.clone();
   }
}