package org.jboss.cache.interceptors;

import org.jboss.cache.InvocationContext;
import org.jboss.cache.batch.BatchContainer;
import org.jboss.cache.commands.VisitableCommand;
import org.jboss.cache.factories.annotations.Inject;
import org.jboss.cache.interceptors.base.CommandInterceptor;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 * Interceptor that captures batched calls and attaches contexts.
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 3.0
 */
public class BatchingInterceptor extends CommandInterceptor
{
   BatchContainer batchContainer;
   TransactionManager transactionManager;

   @Inject
   private void inject(BatchContainer batchContainer, TransactionManager transactionManager)
   {
      this.batchContainer = batchContainer;
      this.transactionManager = transactionManager;
   }

   /**
    * Simply check if there is an ongoing tx.
    * <ul>
    * <li>If there is one, this is a no-op and just passes the call up the chain.</li>
    * <li>If there isn't one and there is a batch in progress, resume the batch's tx, pass up, and finally suspend the batch's tx.</li>
    * <li>If there is no batch in progress, just pass the call up the chain.</li>
    * </ul>
    */
   @Override
   protected Object handleDefault(InvocationContext ctx, VisitableCommand command) throws Throwable
   {
      Transaction tx = null;
      try
      {
         // if in a batch, attach tx
         if (transactionManager.getTransaction() == null &&
               (tx = batchContainer.getBatchTransaction()) != null)
         {
            transactionManager.resume(tx);
         }
         return super.handleDefault(ctx, command);
      }
      finally
      {
         if (tx != null && transactionManager.getTransaction() != null)
            transactionManager.suspend();
      }
   }
}
