package org.jboss.cache.util.concurrent.locks;

import net.jcip.annotations.ThreadSafe;

import java.util.Arrays;
import java.util.concurrent.locks.ReentrantLock;

/**
 * A LockContainer that holds ReentrantLocks
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @see org.jboss.cache.util.concurrent.locks.OwnableReentrantLockContainer
 * @since 3.0
 */
@ThreadSafe
public class ReentrantLockContainer<E> extends LockContainer<E>
{
   ReentrantLock[] sharedLocks;

   /**
    * Creates a new LockContainer which uses a certain number of shared locks across all elements that need to be locked.
    *
    * @param concurrencyLevel concurrency level for number of stripes to create.  Stripes are created in powers of two, with a minimum of concurrencyLevel created.
    */
   public ReentrantLockContainer(int concurrencyLevel)
   {
      initLocks(calculateNumberOfSegments(concurrencyLevel));
   }

   protected void initLocks(int numLocks)
   {
      sharedLocks = new ReentrantLock[numLocks];
      for (int i = 0; i < numLocks; i++) sharedLocks[i] = new ReentrantLock();
   }

   public final ReentrantLock getLock(E object)
   {
      return sharedLocks[hashToIndex(object)];
   }

   public final int getNumLocksHeld()
   {
      int i = 0;
      for (ReentrantLock l : sharedLocks) if (l.isLocked()) i++;
      return i;
   }

   public final boolean ownsLock(E object, Object owner)
   {
      ReentrantLock lock = getLock(object);
      return lock.isHeldByCurrentThread();
   }

   public final boolean isLocked(E object)
   {
      ReentrantLock lock = getLock(object);
      return lock.isLocked();
   }

   public String toString()
   {
      return "ReentrantLockContainer{" +
            "sharedLocks=" + (sharedLocks == null ? null : Arrays.asList(sharedLocks)) +
            '}';
   }

   public void reset()
   {
      initLocks(sharedLocks.length);
   }
}
